<?php

namespace HS\IncomingMail\Mailman;

use hs_notify;
use requestUpdate;
use HS\IncomingMail\Message;
use HS\Domain\Workspace\Request;
use HS\IncomingMail\Processors\LoopCheck;
use HS\IncomingMail\Processors\StaffReply;

class UpdateRequest
{
    private $logger;
    private $mailbox;
    private $mailboxId;

    public function __construct(array $mailbox, $logger)
    {
        $this->logger = $logger;
        $this->mailbox = $mailbox;
        $this->mailboxId = $mailbox['xMailbox'];
    }

    public function save(Message $message, $msgRequestId, $origReq, $msgFiles)
    {
        $msgFromEmail = $message->getFromEmail();

        // See if email is from a system user and apply history appropriately
        $personId = null;
        $user = null;
        if (! empty($msgFromEmail)) {
            $user = apiGetUserByEmail($msgFromEmail);
            if ($user) {
                $personId = $user['xPerson'];
            }
        }

        // Is it an out of control update? Make sure the request history hasn't passed the max threshold.
        if (Request::reachedHistoryLimit($msgRequestId)) {
            $this->logger->display("The MAX request history for $msgRequestId has been reached, message not stored");
            errorLog(sprintf(lg_history_over_limit, $msgRequestId, $msgFromEmail), 'Email Importing', __FILE__, __LINE__);
            return false;
        }

        // Is this in a loop?
        $loopCheck = app(LoopCheck::class)->updateCheck($message, $msgRequestId);
        if ($loopCheck > 0) {
            $this->logger->display('Message flagged as a loop, Loop broken, message not stored');
            //Log note about breaking loop
            errorLog(sprintf(lg_loop_break, $msgRequestId, $msgFromEmail), 'Email Importing', __FILE__, __LINE__);

            return false;
        }

        // First we check to see if this email is from the customer, if so it's public.
        $public = 0;
        if (strtolower($origReq['sEmail']) == strtolower($msgFromEmail)) {
            $public = 1;
            $personId = 0; // If it's staff having their own public request then remove the personid.
        } else {
            // is this a staff notification? <hs-notification header:
            $staffReply = new StaffReply($message, $msgRequestId, $user);
            if ($staffReply->isNotification()) {
                return $staffReply->save($origReq, $this->logger);
            }
        }

        //Hack. Add one second to each so that no 2 requests come in at exactly the same time. This fixes issue with
        //attachments showing up on wrong request note. This will be fixed by adding a new column in a future release.
        $msgDate = date('U') + 1;

        //If request was in trash we should remove it
        if (intval($origReq['fTrash']) === 1) {
            // Reopen request
            $origReq['fTrash'] = 0;
            $origReq['dtGMTTrashed'] = 0;
            $origReq['dtGMTOpened'] = $msgDate;	//current dt

            $update = new requestUpdate($msgRequestId, $origReq, 0, __FILE__, __LINE__);
            $update->notify = false; //notify below instead
            $reqResult = $update->checkChanges();
        }

        // vars for saving:
        $msgMessage = $message->getBody();

        // If the request has been closed then set it to be reopened - give HD option to close again or create new request
        //	based on this message.
        if (intval($origReq['fOpen']) === 0) {
            // Reopen request
            $origReq['fOpen'] = 1;
            $origReq['xStatus'] = hs_setting('cHD_STATUS_ACTIVE', 1);
            $origReq['dtGMTOpened'] = $msgDate;	//current dt
            //if the user isn't active then send to inbox
            $user = apiGetUser($origReq['xPersonAssignedTo']);
            if ($user['fDeleted'] == 1) {
                $origReq['xPersonAssignedTo'] = 0;
            }

            // Add new note. This is done before the triggers so that fullpublichistory is accurate
            $reqHis = apiAddRequestHistory([
                'xPerson' => $personId,
                'xRequest' => $msgRequestId,
                'dtGMTChange' => $msgDate,
                'fPublic' => $public,
                'tNote' => $msgMessage,
                'sRequestHistoryHash' => md5(trim($message->getBody())),
                'fNoteIsHTML' => $message->isHtml(),
                'tEmailHeaders' => hs_serialize($message->headers),
            ]);

            // Do a request update check even if the request is open.
            // https://github.com/UserScape/HelpSpot/pull/1160
            $update = new requestUpdate($msgRequestId, $origReq, 0, __FILE__, __LINE__);
            $update->notify = false; //notify below instead
            $reqResult = $update->checkChanges();

            app('events')->flush('request.history.create');

            //Send notification from here instead of from within addreqhis so that we can send log and body in one email
            if (isset($reqHis) || isset($reqResult['xRequestHistory'])) {
                //Array if both set else just the ID
                if (isset($reqHis) && isset($reqResult['xRequestHistory'])) {
                    $ids = [$reqHis, $reqResult['xRequestHistory']];	//first is body, second is log
                } else {
                    $ids = isset($reqHis) ? $reqHis : $reqResult['xRequestHistory'];
                }

                $notifier = new hs_notify($msgRequestId, $ids, 0, __FILE__, __LINE__);
                $notifier->SetRequestType('existing');
                $notifier->Notify();
            }
        } else {
            $oldReq = apiGetRequest($msgRequestId);

            // Add new note. This is done before the triggers so that fullpublichistory is accurate
            $reqHis = apiAddRequestHistory([
                'xPerson'             => $personId,
                'xRequest'            => $msgRequestId,
                'dtGMTChange'         => $msgDate,
                'fPublic'             => $public,
                'tNote'               => $msgMessage,
                'sRequestHistoryHash' => md5(trim($message->getBody())),
                'fNoteIsHTML'         => $message->isHtml(),
                'tEmailHeaders'       => hs_serialize($message->headers),
            ]);

            //Trigger check
            apiRunTriggers($msgRequestId, $origReq, $oldReq, $msgMessage, $public, $personId, 2, __FILE__, __LINE__);

            app('events')->flush('request.history.create');

            // If request history added then continue
            if ($reqHis) {
                $this->logger->display('Note Added ('.$msgRequestId.')');
                // Insert info about attachments to request_history and write out the files to disk
                apiAddDocument($msgRequestId, $msgDate, $msgFiles, 0, $reqHis, __FILE__, __LINE__);
                $this->logger->display('Finished adding documents');
            }

            // Send staff notifications
            $notifier = new hs_notify($msgRequestId, $reqHis, $personId, __FILE__, __LINE__);
            $notifier->SetRequestType('existing');
            $notifier->Notify();
        }
    }
}
